#!/bin/bash -x
#
# From https://software.intel.com/en-us/forums/software-tuning-performance-optimization-platform-monitoring/topic/535072
#

# define the Integrated Memory Controller performance counter event sets to measure the four events:
#    All Read CAS operations				Event 0x04, Umask 0x03
#    All Write CAS operations				Event 0x04, Umask 0x0C
# Each of the two "SET" variables below includes each of these events on all four memory controller channels.

#SET1='-e "uncore_imc_0/event=0x04,umask=0x03/" -e "uncore_imc_1/event=0x04,umask=0x03/" -e "uncore_imc_2/event=0x04,umask=0x03/" -e "uncore_imc_3/event=0x04,umask=0x03/"'
#SET2='-e "uncore_imc_0/event=0x04,umask=0x0c/" -e "uncore_imc_1/event=0x04,umask=0x0c/" -e "uncore_imc_2/event=0x04,umask=0x0c/" -e "uncore_imc_3/event=0x04,umask=0x0c/"'

PMU_BASE="uncore_imc_"
PMU_COUNT=6
EVT_CODE="0x03"
RD_UMASK="0x01"
WR_UMASK="0x02"

i=0
while [ $i -lt $PMU_COUNT ] ; do
  ARGS="-e ${PMU_BASE}${i}/event=${EVT_CODE},umask=${RD_UMASK}/ $ARGS"
  ARGS="-e ${PMU_BASE}${i}/event=${EVT_CODE},umask=${WR_UMASK}/ $ARGS"
  ((i++))
done

# Since the uncore counters are "per chip", I only need to read these on one core per chip.
# For all of TACC's systems, I know that cores 0 and 9 are on different chips, whichever assignment scheme is used.
#
# "perf stat" flags:
#    "-a" counts for all processes (not just the process run under "perf stat")
#    "-A" tells perf to report results separately for each core, rather than summed
#    "-x ," tells perf to report results as a comma-separated list (easier to import into scripts or spreadsheets)
#    "-o file" directs the output of perf to a separate log file (rather than stdout)

echo "running STREAM bound to socket 0"
export LD_LIBRARY_PATH=/opt/intel/17/linux/compiler/lib/intel64:$LD_LIBRARY_PATH
#perf stat -o perf.out.imc.test1 -x , -a -A -C 0,9 $ARGS numactl --membind=0 --physcpubind=0 ./stream_c.exe
perf stat -x , -a -C 0 $ARGS numactl -m 0 ./stream_c.exe

